#version 430

// outputs from vertex shader
in vec3 vert_world_position;
in vec3 vert_normal;
in vec2 vert_uv;
in vec4 vert_light_pos;
in vec3 light_direction;

layout(location = 20) uniform sampler2D color_texture;
uniform sampler2D depthMap;

layout(location = 21) uniform vec4 base_color;
layout(location = 22) uniform float use_texture;
layout(location = 23) uniform float glow;

layout(location = 30) uniform vec3 light_dir;
layout(location = 31) uniform vec3 light_color;
layout(location = 32) uniform vec3 camera_position;

const vec3 ambient = vec3(0.4);
const vec3 specular_color = vec3(1.0, 1.0, 1.0);
const float shininess = 16.0;
const float brightness_threshold = 1.3;
const float bloom_damping = 0.25;
const float shadow_damping = 0.6;

layout(location = 0) out vec4 frag_color;
layout(location = 1) out vec4 glow_color;
layout(location = 2) out vec4 normal_color;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
	
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
	
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(depthMap, projCoords.xy).r;
	
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
	
    // check whether current frag pos is in shadow
	// bias against shadow acne
	float bias = 0.005;
	//float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(depthMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
			float pcfDepth = texture(depthMap, projCoords.xy + vec2(x, y) * texelSize).r; 
			shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
		}    
	}
	
	shadow /= 9.0;
	
	if(projCoords.z > 1.0)
        shadow = 0.0;
	
    return shadow;
}

void main() {

  vec3 normal = normalize(vert_normal);
  
  float diffuse = max(dot(light_dir, normal), 0.0);
  float specular = 0.0;
  
  float shadow = ShadowCalculation(vert_light_pos)*shadow_damping;
  
  if (diffuse > 0.0) {
    vec3 view_dir = normalize(camera_position - vert_world_position);
    vec3 half_dir = normalize(light_dir + view_dir);
    specular = pow(max(dot(half_dir, normal), 0.0), shininess);
  }

  vec4 tex_color = texture(color_texture, vert_uv);
  vec3 light = (1.0 - shadow) * ((diffuse * light_color) + (specular * specular_color)) + ambient;
 
  // quantize intensity for cel shading, 
  float numShades = 3;
  float intensity = diffuse + specular + 0.3; 
  float shadeIntensity = ceil(intensity * numShades)/ numShades;
 
  vec3 color = light * mix(base_color.rgb, tex_color.rgb, min(tex_color.a, use_texture));
  frag_color = vec4(color * shadeIntensity, mix(base_color.a, tex_color.a+base_color.a, use_texture));
  
  float brightness = dot(frag_color.rgb, vec3(0.2126, 0.7152, 0.0722));
  glow_color = mix(vec4(color.rgb, glow), vec4(color.rgb, bloom_damping), step(brightness_threshold, brightness));
  
  normal_color = vec4(base_color.rgb, frag_color.a);
  //normal_color = vec4(frag_color.rgb*0.4 + normal*0.6, frag_color.a);
  //normal_color = vec4(vec3(gl_FragCoord.z / gl_FragCoord.w / 50.0)*0.3 + normal*0.5, 1.0);
  //normal_color = vec4(vert_world_position*0.3+normal*0.7, 1.0);
}